@@ -41,11 +41,6 @@ except ImportError:
             'install paths may be incorrect', file=sys.stderr)
 
 windows_build = os.name == 'nt'
-if windows_build:
-    import cx_Freeze
-    command_base = cx_Freeze
-else:
-    command_base = distutils
 
 
 def has_help(self):
@@ -64,7 +59,7 @@ def has_data(self):
     return "build_data" in self.distribution.cmdclass
 
 
-command_base.command.build.Build.sub_commands.extend([
+distutils.command.build.build.sub_commands.extend([
     ("build_i18n", has_i18n),
     ("build_icons", has_icons),
     ("build_help", has_help),
@@ -87,7 +82,7 @@ class MeldDistribution(distutils.dist.Distribution):
 class build_data(distutils.cmd.Command):
 
     gschemas = [
-        ('share/glib-2.0/schemas', ['data/org.gnome.meld.gschema.xml'])
+        ('share/glib-2.0/schemas', ['data/org.gnome.Meld.gschema.xml'])
     ]
 
     frozen_gschemas = [
@@ -372,12 +367,12 @@ class build_i18n(distutils.cmd.Command):
         # Meson-based initial variable templating in distutils.
         import shutil
         shutil.copyfile(
-            'data/org.gnome.meld.desktop.in.in',
-            'data/org.gnome.meld.desktop.in',
+            'data/org.gnome.Meld.desktop.in.in',
+            'data/org.gnome.Meld.desktop.in',
         )
         shutil.copyfile(
-            'data/org.gnome.meld.appdata.xml.in.in',
-            'data/org.gnome.meld.appdata.xml.in',
+            'data/org.gnome.Meld.appdata.xml.in.in',
+            'data/org.gnome.Meld.appdata.xml.in',
         )
 
         self._rebuild_po()
