@@ -13,6 +13,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import sys
+from typing import Optional
+
 from gi.repository import Gio, GObject, GtkSource, Pango
 
 import meld.conf
@@ -33,6 +36,7 @@ class MeldSettings(GObject.GObject):
         self.on_setting_changed(settings, 'filename-filters')
         self.on_setting_changed(settings, 'text-filters')
         self.on_setting_changed(settings, 'use-system-font')
+        self.on_setting_changed(settings, 'prefer-dark-theme')
         self.style_scheme = self._style_scheme_from_gsettings()
         settings.connect('changed', self.on_setting_changed)
 
@@ -48,7 +52,7 @@ class MeldSettings(GObject.GObject):
         elif key in ('use-system-font', 'custom-font'):
             self.font = self._current_font_from_gsetting()
             self.emit('changed', 'font')
-        elif key in ('style-scheme'):
+        elif key in ('style-scheme', 'prefer-dark-theme'):
             self.style_scheme = self._style_scheme_from_gsettings()
             self.emit('changed', 'style-scheme')
 
@@ -56,7 +60,8 @@ class MeldSettings(GObject.GObject):
         from meld.style import set_base_style_scheme
         manager = GtkSource.StyleSchemeManager.get_default()
         scheme = manager.get_scheme(settings.get_string('style-scheme'))
-        set_base_style_scheme(scheme)
+        prefer_dark = settings.get_boolean("prefer-dark-theme")
+        set_base_style_scheme(scheme, prefer_dark)
         return scheme
 
     def _filters_from_gsetting(self, key, filt_type):
@@ -69,7 +74,13 @@ class MeldSettings(GObject.GObject):
 
     def _current_font_from_gsetting(self, *args):
         if settings.get_boolean('use-system-font'):
-            font_string = interface_settings.get_string('monospace-font-name')
+            if sys.platform == 'win32':
+                font_string = 'Consolas 11'
+            elif interface_settings:
+                font_string = interface_settings.get_string(
+                    'monospace-font-name')
+            else:
+                font_string = 'monospace'
         else:
             font_string = settings.get_string('custom-font')
         return Pango.FontDescription(font_string)
@@ -90,11 +101,23 @@ def load_settings_schema(schema_id):
     return settings
 
 
+def load_interface_settings() -> Optional[Gio.Settings]:
+
+    # We conditionally load these since they're only used for default
+    # fonts and can sometimes be missing (e.g., in some Windows setups)
+    default_source = Gio.SettingsSchemaSource.get_default()
+    schema = default_source.lookup("org.gnome.desktop.interface", False)
+    if not schema:
+        return None
+
+    return Gio.Settings.new_full(schema=schema, backend=None, path=None)
+
+
 def create_settings():
     global settings, interface_settings, _meldsettings
 
     settings = load_settings_schema(meld.conf.SETTINGS_SCHEMA_ID)
-    interface_settings = Gio.Settings.new('org.gnome.desktop.interface')
+    interface_settings = load_interface_settings()
     _meldsettings = MeldSettings()
 
 
@@ -113,6 +136,10 @@ def bind_settings(obj):
         settings.bind(settings_id, obj, property_id, bind_flags)
 
 
+def get_settings() -> Gio.Settings:
+    return settings
+
+
 def get_meld_settings() -> MeldSettings:
     return _meldsettings
 
