@@ -41,6 +41,7 @@ from meld.vc._vc import (  # noqa: F401
     STATE_NONEXIST,
     STATE_NORMAL,
     STATE_REMOVED,
+    STATE_SPINNER,
 )
 
 _GIGtk = None
@@ -71,10 +72,6 @@ class DiffTreeStore(SearchableTreeStore):
         self.ntree = ntree
         self._setup_default_styles()
 
-    def on_style_updated(self, widget):
-        style = widget.get_style_context()
-        self._setup_default_styles(style)
-
     def _setup_default_styles(self, style=None):
         roman, italic = Pango.Style.NORMAL, Pango.Style.ITALIC
         normal, bold = Pango.Weight.NORMAL, Pango.Weight.BOLD
@@ -102,23 +99,25 @@ class DiffTreeStore(SearchableTreeStore):
             (del_fg, roman,  bold,   True),  # STATE_REMOVED
             (del_fg, roman,  bold,   True),  # STATE_MISSING
             (unk_fg, roman,  normal, True),  # STATE_NONEXIST
+            (None,   italic, normal, None),  # STATE_SPINNER
         ]
 
         self.icon_details = [
-            # file-icon, folder-icon, file-tint, folder-tint
-            ("text-x-generic", "folder", None,   None),    # IGNORED
-            ("text-x-generic", "folder", None,   None),    # NONE
-            ("text-x-generic", "folder", None,   None),    # NORMAL
-            ("text-x-generic", "folder", None,   None),    # NOCHANGE
-            ("dialog-warning", None,     None,   None),    # ERROR
-            (None,             None,     None,   None),    # EMPTY
-            ("text-x-generic", "folder", new_fg, None),    # NEW
-            ("text-x-generic", "folder", mod_fg, None),    # MODIFIED
-            ("text-x-generic", "folder", mod_fg, None),    # RENAMED
-            ("text-x-generic", "folder", con_fg, None),    # CONFLICT
-            ("text-x-generic", "folder", del_fg, None),    # REMOVED
-            ("text-x-generic", "folder", unk_fg, unk_fg),  # MISSING
-            ("text-x-generic", "folder", unk_fg, unk_fg),  # NONEXIST
+            # file-icon, folder-icon, file-tint
+            ("text-x-generic", "folder", None),    # IGNORED
+            ("text-x-generic", "folder", None),    # NONE
+            ("text-x-generic", "folder", None),    # NORMAL
+            ("text-x-generic", "folder", None),    # NOCHANGE
+            ("dialog-warning-symbolic", None, None),    # ERROR
+            (None, None, None),    # EMPTY
+            ("text-x-generic", "folder", new_fg),    # NEW
+            ("text-x-generic", "folder", mod_fg),    # MODIFIED
+            ("text-x-generic", "folder", mod_fg),    # RENAMED
+            ("text-x-generic", "folder", con_fg),    # CONFLICT
+            ("text-x-generic", "folder", del_fg),    # REMOVED
+            (None, "folder", unk_fg),  # MISSING
+            (None, "folder", unk_fg),  # NONEXIST
+            ("text-x-generic", "folder", None),    # SPINNER
         ]
 
         assert len(self.icon_details) == len(self.text_attributes) == STATE_MAX
@@ -166,7 +165,7 @@ class DiffTreeStore(SearchableTreeStore):
 
     def set_state(self, it, pane, state, label, isdir=0):
         icon = self.icon_details[state][1 if isdir else 0]
-        tint = self.icon_details[state][3 if isdir else 2]
+        tint = None if isdir else self.icon_details[state][2]
         fg, style, weight, strike = self.text_attributes[state]
         self.unsafe_set(it, pane, {
             COL_STATE: str(state),
