@@ -24,6 +24,7 @@
 
 import collections
 import itertools
+import logging
 import os
 import re
 import shutil
@@ -34,6 +35,9 @@ from typing import ClassVar
 from gi.repository import Gio, GLib
 
 from meld.conf import _
+from meld.misc import get_hide_window_startupinfo
+
+log = logging.getLogger(__name__)
 
 # ignored, new, normal, ignored changes,
 # error, placeholder, vc added
@@ -42,7 +46,7 @@ from meld.conf import _
 (STATE_IGNORED, STATE_NONE, STATE_NORMAL, STATE_NOCHANGE,
     STATE_ERROR, STATE_EMPTY, STATE_NEW,
     STATE_MODIFIED, STATE_RENAMED, STATE_CONFLICT, STATE_REMOVED,
-    STATE_MISSING, STATE_NONEXIST, STATE_MAX) = list(range(14))
+    STATE_MISSING, STATE_NONEXIST, STATE_SPINNER, STATE_MAX) = list(range(15))
 
 # VC conflict types
 (CONFLICT_MERGED, CONFLICT_BASE, CONFLICT_LOCAL,
@@ -79,6 +83,7 @@ class Entry:
         STATE_REMOVED: _("Removed"),
         STATE_MISSING: _("Missing"),
         STATE_NONEXIST: _("Not present"),
+        STATE_SPINNER: _("Scanning…"),
     }
 
     def __init__(self, path, name, state, isdir, options=None):
@@ -162,7 +167,9 @@ class Vc:
         cmd = (self.CMD,) + args
         return subprocess.Popen(
             cmd, cwd=self.location, stdout=subprocess.PIPE,
-            universal_newlines=use_locale_encoding)
+            universal_newlines=use_locale_encoding,
+            startupinfo=get_hide_window_startupinfo(),
+        )
 
     def get_files_to_commit(self, paths):
         """Get a list of files that will be committed from paths
@@ -278,9 +285,20 @@ class Vc:
 
     def get_entries(self, base):
         parent = Gio.File.new_for_path(base)
-        enumerator = parent.enumerate_children(
-            'standard::name,standard::display-name,standard::type',
-            Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS, None)
+        try:
+            enumerator = parent.enumerate_children(
+                'standard::name,standard::display-name,standard::type',
+                Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
+                None,
+            )
+        except GLib.Error as err:
+            if err.matches(
+                Gio.io_error_quark(),
+                Gio.IOErrorEnum.PERMISSION_DENIED
+            ):
+                log.error(f"Failed to scan folder {base!r}; permission denied")
+                return
+            raise
 
         for file_info in enumerator:
             if file_info.get_name() == self.VC_DIR:
@@ -407,9 +425,12 @@ def popen(cmd, cwd=None, use_locale_encoding=True):
     text stream with universal newlines.
     If use_locale_encoding is False output is treated as binary stream.
     """
+
     process = subprocess.Popen(
         cmd, cwd=cwd, stdout=subprocess.PIPE,
-        universal_newlines=use_locale_encoding)
+        universal_newlines=use_locale_encoding,
+        startupinfo=get_hide_window_startupinfo(),
+    )
     return process.stdout
 
 
@@ -426,7 +447,9 @@ def call_temp_output(cmd, cwd, file_id='', suffix=None):
     of the temporary file's name.
     """
     process = subprocess.Popen(
-        cmd, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        cmd, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+        startupinfo=get_hide_window_startupinfo(),
+    )
     vc_file = process.stdout
 
     # Error handling here involves doing nothing; in most cases, the only
@@ -442,7 +465,10 @@ def call_temp_output(cmd, cwd, file_id='', suffix=None):
 # Return the return value of a given command
 def call(cmd, cwd=None):
     devnull = open(os.devnull, "wb")
-    return subprocess.call(cmd, cwd=cwd, stdout=devnull, stderr=devnull)
+    return subprocess.call(
+        cmd, cwd=cwd, stdout=devnull, stderr=devnull,
+        startupinfo=get_hide_window_startupinfo(),
+    )
 
 
 base_re = re.compile(
