@@ -28,7 +28,7 @@ import unicodedata
 from collections import namedtuple
 from decimal import Decimal
 from mmap import ACCESS_COPY, mmap
-from typing import DefaultDict, List, NamedTuple, Optional, Tuple
+from typing import DefaultDict, Dict, List, NamedTuple, Optional, Tuple
 
 from gi.repository import Gdk, Gio, GLib, GObject, Gtk
 
@@ -36,8 +36,9 @@ from gi.repository import Gdk, Gio, GLib, GObject, Gtk
 from meld import misc, tree
 from meld.conf import _
 from meld.const import FILE_FILTER_ACTION_FORMAT, MISSING_TIMESTAMP
+from meld.externalhelpers import open_files_external
 from meld.iohelpers import find_shared_parent_path, trash_or_confirm
-from meld.melddoc import MeldDoc, open_files_external
+from meld.melddoc import MeldDoc
 from meld.misc import all_same, apply_text_filters, with_focused_pane
 from meld.recent import RecentType
 from meld.settings import bind_settings, get_meld_settings, settings
@@ -65,10 +66,14 @@ class StatItem(namedtuple('StatItem', 'mode size time')):
         return StatItem(stat.S_IFMT(stat_result.st_mode),
                         stat_result.st_size, stat_result.st_mtime)
 
-    def shallow_equal(self, other, time_resolution_ns):
+    def shallow_equal(self, other: "StatItem", time_resolution_ns: int) -> bool:
         if self.size != other.size:
             return False
 
+        # Check for the ignore-timestamp configuration first
+        if time_resolution_ns == -1:
+            return True
+
         # Shortcut to avoid expensive Decimal calculations. 2 seconds is our
         # current accuracy threshold (for VFAT), so should be safe for now.
         if abs(self.time - other.time) > 2:
@@ -249,16 +254,15 @@ def _files_same(files, regexes, comparison_args):
 
 EMBLEM_NEW = "emblem-new"
 EMBLEM_SELECTED = "emblem-default-symbolic"
-EMBLEM_SYMLINK = "emblem-symbolic-link"
 
-COL_EMBLEM, COL_EMBLEM_SECONDARY, COL_SIZE, COL_TIME, COL_PERMS, COL_END = (
-    range(tree.COL_END, tree.COL_END + 6))
+COL_EMBLEM, COL_SIZE, COL_TIME, COL_PERMS, COL_END = (
+    range(tree.COL_END, tree.COL_END + 5))
 
 
 class DirDiffTreeStore(tree.DiffTreeStore):
     def __init__(self, ntree):
         # FIXME: size should be a GObject.TYPE_UINT64, but we use -1 as a flag
-        super().__init__(ntree, [str, str, GObject.TYPE_INT64, float, int])
+        super().__init__(ntree, [str, GObject.TYPE_INT64, float, int])
 
     def add_error(self, parent, msg, pane):
         defaults = {
@@ -284,11 +288,15 @@ class CanonicalListing:
     """Multi-pane lists with canonicalised matching and error detection"""
 
     items: DefaultDict[str, List[Optional[str]]]
+    stripped_items: Dict[str, str]
     errors: List[Tuple[int, str, str]]
+    whitespace: List[Tuple[int, str]]
 
     def __init__(self, n: int, options: ComparisonOptions):
         self.items = collections.defaultdict(lambda: [None] * n)
+        self.stripped_items = {}
         self.errors = []
+        self.whitespace = []
         self.options = options
 
     def add(self, pane: int, item: str):
@@ -308,6 +316,15 @@ class CanonicalListing:
         else:
             self.errors.append((pane, item, existing_item))
 
+        stripped_item = ci.strip()
+        if stripped_item in self.stripped_items:
+            # If we have an existing stripped item and its pre-stripping
+            # value differs, then we have a case of misleading whitespace
+            if self.stripped_items[stripped_item] != ci:
+                self.whitespace.append((pane, item))
+        else:
+            self.stripped_items[stripped_item] = ci
+
     def get(self):
         def filled(seq):
             fill_value = next(s for s in seq if s)
@@ -360,7 +377,7 @@ class ComparisonMarker(NamedTuple):
 
 
 @Gtk.Template(resource_path='/org/gnome/meld/ui/dirdiff.ui')
-class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
+class DirDiff(Gtk.Box, tree.TreeviewCommon, MeldDoc):
 
     __gtype_name__ = "DirDiff"
 
@@ -466,6 +483,15 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         tree.STATE_MODIFIED: ("modified", "folder-status-modified"),
     }
 
+    replaced_entries = (
+        # Remove Ctrl+Page Up/Down bindings. These are used to do horizontal
+        # scrolling in GTK by default, but we preference easy tab switching.
+        (Gdk.KEY_Page_Up, Gdk.ModifierType.CONTROL_MASK),
+        (Gdk.KEY_KP_Page_Up, Gdk.ModifierType.CONTROL_MASK),
+        (Gdk.KEY_Page_Down, Gdk.ModifierType.CONTROL_MASK),
+        (Gdk.KEY_KP_Page_Down, Gdk.ModifierType.CONTROL_MASK),
+    )
+
     def __init__(self, num_panes):
         super().__init__()
         # FIXME:
@@ -479,6 +505,12 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         MeldDoc.__init__(self)
         bind_settings(self)
 
+        binding_set_names = ("GtkScrolledWindow", "GtkTreeView")
+        for set_name in binding_set_names:
+            binding_set = Gtk.binding_set_find(set_name)
+            for key, modifiers in self.replaced_entries:
+                Gtk.binding_entry_remove(binding_set, key, modifiers)
+
         self.view_action_group = Gio.SimpleActionGroup()
 
         property_actions = (
@@ -514,6 +546,7 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             self.view_action_group.add_action(action)
 
         actions = (
+            ("folder-filter", None, GLib.Variant.new_boolean(False)),
             ("folder-status-same", self.action_filter_state_change,
                 GLib.Variant.new_boolean(False)),
             ("folder-status-new", self.action_filter_state_change,
@@ -527,7 +560,8 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         )
         for (name, callback, state) in actions:
             action = Gio.SimpleAction.new_stateful(name, None, state)
-            action.connect('change-state', callback)
+            if callback:
+                action.connect("change-state", callback)
             self.view_action_group.add_action(action)
 
         builder = Gtk.Builder.new_from_resource(
@@ -584,9 +618,6 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         self.custom_labels = []
         self.set_num_panes(num_panes)
 
-        self.connect("style-updated", self.model.on_style_updated)
-        self.model.on_style_updated(self)
-
         self.do_to_others_lock = False
         for treeview in self.treeview:
             treeview.set_search_equal_func(tree.treeview_search_cb, None)
@@ -615,7 +646,6 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
                 renicon,
                 icon_name=col_index(tree.COL_ICON, i),
                 emblem_name=col_index(COL_EMBLEM, i),
-                secondary_emblem_name=col_index(COL_EMBLEM_SECONDARY, i),
                 icon_tint=col_index(tree.COL_TINT, i)
             )
             self.treeview[i].append_column(column)
@@ -852,18 +882,18 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
 
         parent_path = find_shared_parent_path(self.folders)
         for pane, folder in enumerate(self.folders):
-            self.folder_label[pane].set_file(folder)
-            self.folder_label[pane].set_parent_file(parent_path)
+            self.folder_label[pane].gfile = folder
+            self.folder_label[pane].parent_gfile = parent_path
             self.folder_open_button[pane].props.file = folder
 
         # This is difficult to trigger, and to test. Most of the time here we
         # will actually have had UTF-8 from GTK, which has been unicode-ed by
         # the time we get this far. This is a fallback, and may be wrong!
         locations = list(locations)
-        for i, l in enumerate(locations):
-            if l and not isinstance(l, str):
-                locations[i] = l.decode(sys.getfilesystemencoding())
-        locations = [os.path.abspath(l) if l else '' for l in locations]
+        for i, location in enumerate(locations):
+            if location and not isinstance(location, str):
+                locations[i] = location.decode(sys.getfilesystemencoding())
+        locations = [os.path.abspath(loc) if loc else '' for loc in locations]
 
         self.current_path = None
         self.marked = None
@@ -871,7 +901,7 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         for m in self.msgarea_mgr:
             m.clear()
         child = self.model.add_entries(None, locations)
-        self.treeview0.grab_focus()
+        self.on_treeview_focus_in_event(self.treeview0, None)
         self._update_item_state(child)
         self.recompute_label()
         self.scheduler.remove_all_tasks()
@@ -903,7 +933,6 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             self.model.set_state(it, pane, tree.STATE_SPINNER, label, True)
             self.model.unsafe_set(it, pane, {
                 COL_EMBLEM: None,
-                COL_EMBLEM_SECONDARY: None,
                 COL_TIME: MISSING_TIMESTAMP,
                 COL_SIZE: -1,
                 COL_PERMS: -1
@@ -943,6 +972,7 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
 
         shadowed_entries = []
         invalid_filenames = []
+        whitespace_filenames = []
 
         # TODO: Map these action states to GObject props instead?
         comparison_options = ComparisonOptions(
@@ -1037,6 +1067,9 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             for pane, f1, f2 in dirs.errors + files.errors:
                 shadowed_entries.append((pane, roots[pane], f1, f2))
 
+            for pane, f in dirs.whitespace + files.whitespace:
+                whitespace_filenames.append((pane, roots[pane], f))
+
             alldirs = self._filter_on_state(roots, dirs.get())
             allfiles = self._filter_on_state(roots, files.get())
 
@@ -1087,8 +1120,10 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
                 expanded.add(tree_path_as_tuple(path))
 
         duplicate_dirs = list(set(p for p in roots if roots.count(p) > 1))
-        if invalid_filenames or shadowed_entries:
-            self._show_tree_wide_errors(invalid_filenames, shadowed_entries)
+        if any((invalid_filenames, shadowed_entries, whitespace_filenames)):
+            self._show_tree_wide_errors(
+                invalid_filenames, shadowed_entries, whitespace_filenames
+            )
         elif duplicate_dirs:
             # Since we can only load 3 dirs we can have at most 1 duplicate
             self._show_duplicate_directory(duplicate_dirs[0])
@@ -1106,7 +1141,7 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             self._update_item_state(self.model.get_iter(rootpath))
 
         self.force_cursor_recalculate = True
-        self.treeview[0].set_cursor(Gtk.TreePath.new_first())
+        self.treeview[0].set_cursor(rootpath)
 
     def _show_duplicate_directory(self, duplicate_directory):
         for index in range(self.num_panes):
@@ -1141,7 +1176,7 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
 
         for pane in range(self.num_panes):
             msgarea = self.msgarea_mgr[pane].new_from_text_and_icon(
-                'dialog-information-symbolic', primary, secondary)
+                primary, secondary)
             button = msgarea.add_button(_("Hide"), Gtk.ResponseType.CLOSE)
             if pane == 0:
                 button.props.label = _("Hi_de")
@@ -1152,7 +1187,9 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             msgarea.connect("response", clear_all)
             msgarea.show_all()
 
-    def _show_tree_wide_errors(self, invalid_filenames, shadowed_entries):
+    def _show_tree_wide_errors(
+        self, invalid_filenames, shadowed_entries, whitespace_filenames
+    ) -> None:
         header = _("Multiple errors occurred while scanning this folder")
         invalid_header = _("Files with invalid encodings found")
         # TRANSLATORS: This is followed by a list of files
@@ -1163,6 +1200,12 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         shadowed_secondary = _("You are running a case insensitive comparison "
                                "on a case sensitive filesystem. The following "
                                "files in this folder are hidden:")
+        whitespace_header = _("Files had mismatched leading or trailing whitespace")
+        # TRANSLATORS: This is followed by a list of files
+        whitespace_secondary = _(
+            "This comparison found some files that differed only in leading or "
+            "trailing whitespace. Their names appear as:"
+        )
 
         invalid_entries = [[] for i in range(self.num_panes)]
         for pane, root, f in invalid_filenames:
@@ -1177,24 +1220,42 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             )
             formatted_entries[pane].append(entry_str)
 
-        if invalid_filenames or shadowed_entries:
+        whitespace_entries = [[] for i in range(self.num_panes)]
+        for _pane, root, f in whitespace_filenames:
+            for pane in range(self.num_panes):
+                whitespace_entries[pane].append(os.path.join(root, f))
+
+        if invalid_filenames or shadowed_entries or whitespace_filenames:
             for pane in range(self.num_panes):
                 invalid = "\n".join(invalid_entries[pane])
                 shadowed = "\n".join(formatted_entries[pane])
-                if invalid and shadowed:
-                    messages = (invalid_secondary, invalid, "",
-                                shadowed_secondary, shadowed)
-                elif invalid:
-                    header = invalid_header
-                    messages = (invalid_secondary, invalid)
-                elif shadowed:
-                    header = shadowed_header
-                    messages = (shadowed_secondary, shadowed)
-                else:
+                whitespace = "\n".join(whitespace_entries[pane])
+
+                error_type_count = [
+                    bool(err) for err in (invalid, shadowed, whitespace)
+                ].count(True)
+                if error_type_count == 0:
                     continue
-                secondary = "\n".join(messages)
+                elif error_type_count == 1:
+                    if invalid:
+                        header = invalid_header
+                    elif shadowed:
+                        header = shadowed_header
+                    elif whitespace:
+                        header = whitespace_header
+
+                messages = []
+                if invalid:
+                    messages.extend([invalid_secondary, invalid, ""])
+                if shadowed:
+                    messages.extend([shadowed_secondary, shadowed, ""])
+                if whitespace:
+                    messages.extend([whitespace_secondary, whitespace, ""])
+
+                secondary = ("\n".join(messages)).strip()
                 self.msgarea_mgr[pane].add_dismissable_msg(
-                    'dialog-error-symbolic', header, secondary)
+                    "dialog-error-symbolic", header, secondary
+                )
 
     def copy_selected(self, direction):
         assert direction in (-1, 1)
@@ -1448,7 +1509,8 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
     @Gtk.Template.Callback()
     def on_treeview_row_activated(self, view, path, column):
         pane = self.treeview.index(view)
-        rows = self.model.value_paths(self.model.get_iter(path))
+        it = self.model.get_iter(path)
+        rows = self.model.value_paths(it)
         # Click a file: compare; click a directory: expand; click a missing
         # entry: check the next neighbouring entry
         pane_ordering = ((0, 1, 2), (1, 2, 0), (2, 1, 0))
@@ -1459,11 +1521,7 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         if not rows[pane]:
             return
         if os.path.isfile(rows[pane]):
-            diff_gfiles = [
-                Gio.File.new_for_path(r) if os.path.isfile(r) else None
-                for r in rows
-            ]
-            self.create_diff_signal.emit(diff_gfiles, {})
+            self.run_diff_from_iter(it)
         elif os.path.isdir(rows[pane]):
             if view.row_expanded(path):
                 view.collapse_row(path)
@@ -1491,9 +1549,11 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         tree.emit("cursor-changed")
 
     def run_diff_from_iter(self, it):
-        row_paths = self.model.value_paths(it)
-        gfiles = [Gio.File.new_for_path(p)
-                  for p in row_paths if os.path.exists(p)]
+        rows = self.model.value_paths(it)
+        gfiles = [
+            Gio.File.new_for_path(r) if os.path.isfile(r) else None
+            for r in rows
+        ]
         self.create_diff_signal.emit(gfiles, {})
 
     def action_diff(self, *args):
@@ -1596,9 +1656,8 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             self.model.value_path(self.model.get_iter(p), pane)
             for p in self._get_selected_paths(pane)
         ]
-        files = [f for f in files if f]
-        if files:
-            open_files_external(files)
+        gfiles = [Gio.File.new_for_path(f) for f in files if f]
+        open_files_external(gfiles)
 
     def action_copy_file_paths(self, *args):
         pane = self._get_focused_pane()
@@ -1658,28 +1717,39 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
                roots - array of root directories
                fileslist - array of filename tuples of length len(roots)
         """
-        assert len(roots) == self.model.ntree
         ret = []
         regexes = [f.byte_filter for f in self.text_filters if f.active]
         for files in fileslist:
             curfiles = [os.path.join(r, f) for r, f in zip(roots, files)]
             is_present = [os.path.exists(f) for f in curfiles]
-            all_present = 0 not in is_present
-            if all_present:
+            if all(is_present):
                 comparison_result = self.file_compare(curfiles, regexes)
-                if comparison_result in (
-                        Same, DodgySame):
-                    state = tree.STATE_NORMAL
+                if comparison_result in (Same, DodgySame):
+                    states = {tree.STATE_NORMAL}
                 elif comparison_result == SameFiltered:
-                    state = tree.STATE_NOCHANGE
+                    states = {tree.STATE_NOCHANGE}
                 else:
-                    state = tree.STATE_MODIFIED
+                    states = {tree.STATE_MODIFIED}
+            elif is_present.count(True) > 1:
+                # In a three-way comparison, we can have files in e.g., pane
+                # 1 and 2 be different to each other, and there be no file in
+                # pane 3. This row should be considered both modified (1 -> 2)
+                # and new (2 -> 3).
+                curfiles = [
+                    f for f, exists in zip(curfiles, is_present) if exists
+                ]
+                comparison_result = self.file_compare(curfiles, regexes)
+                if comparison_result in (Same, DodgySame, SameFiltered):
+                    states = {tree.STATE_NEW}
+                else:
+                    states = {tree.STATE_NEW, tree.STATE_MODIFIED}
             else:
-                state = tree.STATE_NEW
+                states = {tree.STATE_NEW}
             # Always retain NORMAL folders for comparison; we remove these
             # later if they have no children.
-            if (state in self.state_filters or
-                    all(os.path.isdir(f) for f in curfiles)):
+            all_folders = all(os.path.isdir(f) for f in curfiles)
+            states_match_filters = bool(states & set(self.state_filters))
+            if states_match_filters or all_folders:
                 ret.append(files)
         return ret
 
@@ -1769,13 +1839,15 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
                 else:
                     emblem = EMBLEM_NEW if j in newest else None
 
-                link_emblem = EMBLEM_SYMLINK if j in symlinks else None
                 self.model.unsafe_set(it, j, {
                     COL_EMBLEM: emblem,
-                    COL_EMBLEM_SECONDARY: link_emblem,
                     COL_TIME: times[j],
                     COL_PERMS: perms[j]
                 })
+                if j in symlinks:
+                    self.model.unsafe_set(it, j, {
+                        tree.COL_ICON: "symbolic-link-symbolic",
+                    })
                 # Size is handled independently, because unsafe_set
                 # can't correctly box GObject.TYPE_INT64.
                 self.model.set(
@@ -1842,6 +1914,11 @@ class DirDiff(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
 
     def set_labels(self, labels):
         labels = labels[:self.num_panes]
+
+        for label, flabel in zip(labels, self.folder_label):
+            if label:
+                flabel.props.custom_label = label
+
         extra = self.num_panes - len(labels)
         if extra:
             labels.extend([""] * extra)
