@@ -16,7 +16,7 @@
 
 import logging
 import os
-from typing import Optional, Sequence
+from typing import Any, Dict, Optional, Sequence
 
 from gi.repository import Gdk, Gio, GLib, Gtk
 
@@ -31,8 +31,10 @@ from meld.const import (
 )
 from meld.dirdiff import DirDiff
 from meld.filediff import FileDiff
+from meld.imagediff import ImageDiff, files_are_images
 from meld.melddoc import ComparisonState, MeldDoc
 from meld.menuhelpers import replace_menu_section
+from meld.misc import guess_if_remote_x11
 from meld.newdifftab import NewDiffTab
 from meld.recent import RecentType, recent_comparisons
 from meld.settings import get_meld_settings
@@ -92,8 +94,9 @@ class MeldWindow(Gtk.ApplicationWindow):
         # Initialise sensitivity for important actions
         self.lookup_action('stop').set_enabled(False)
 
-        # Fake out the spinner on Windows. See Gitlab issue #133.
-        if os.name == 'nt':
+        # Fake out the spinner on Windows or X11 forwarding. See Gitlab
+        # issues #133 and #507.
+        if os.name == "nt" or guess_if_remote_x11():
             for attr in ('stop', 'hide', 'show', 'start'):
                 setattr(self.spinner, attr, lambda *args: True)
 
@@ -208,19 +211,26 @@ class MeldWindow(Gtk.ApplicationWindow):
 
     @Gtk.Template.Callback()
     def on_delete_event(self, *extra):
-        should_cancel = False
         # Delete pages from right-to-left.  This ensures that if a version
         # control page is open in the far left page, it will be closed last.
+        responses = []
         for page in reversed(self.notebook.get_children()):
             self.notebook.set_current_page(self.notebook.page_num(page))
-            response = page.on_delete_event()
-            if response == Gtk.ResponseType.CANCEL:
-                should_cancel = True
+            responses.append(page.on_delete_event())
 
-        should_cancel = should_cancel or self.has_pages()
-        if should_cancel:
-            self.should_close = True
-        return should_cancel
+        have_cancelled_tabs = any(r == Gtk.ResponseType.CANCEL for r in responses)
+        have_saving_tabs = any(r == Gtk.ResponseType.APPLY for r in responses)
+
+        # If we have tabs that are not straight OK responses, we cancel the
+        # close. Either something has cancelled the close, or we temporarily
+        # cancel the close while async saving is happening.
+        cancel_delete = have_cancelled_tabs or have_saving_tabs or self.has_pages()
+        # If we have only saving and no cancelled tabs, we record that we
+        # should close once the other tabs have closed (assuming the state)
+        # doesn't otherwise change.
+        self.should_close = have_saving_tabs and not have_cancelled_tabs
+
+        return cancel_delete
 
     def has_pages(self):
         return self.notebook.get_n_pages() > 0
@@ -357,6 +367,7 @@ class MeldWindow(Gtk.ApplicationWindow):
         assert len(gfiles) in (1, 2, 3)
         doc = DirDiff(len(gfiles))
         self._append_page(doc)
+        gfiles = [f or Gio.File.new_for_path("") for f in gfiles]
         doc.folders = gfiles
         doc.set_locations()
         if auto_compare:
@@ -366,7 +377,12 @@ class MeldWindow(Gtk.ApplicationWindow):
     def append_filediff(
             self, gfiles, *, encodings=None, merge_output=None, meta=None):
         assert len(gfiles) in (1, 2, 3)
-        doc = FileDiff(len(gfiles))
+
+        # Check whether to show image window or not.
+        if files_are_images(gfiles):
+            doc = ImageDiff(len(gfiles))
+        else:
+            doc = FileDiff(len(gfiles))
         self._append_page(doc)
         doc.set_files(gfiles, encodings)
         if merge_output is not None:
@@ -388,13 +404,21 @@ class MeldWindow(Gtk.ApplicationWindow):
             doc.set_merge_output_file(merge_output)
         return doc
 
-    def append_diff(self, gfiles, auto_compare=False, auto_merge=False,
-                    merge_output=None, meta=None):
+    def append_diff(
+        self,
+        gfiles: Sequence[Optional[Gio.File]],
+        auto_compare: bool = False,
+        auto_merge: bool = False,
+        merge_output: Optional[Gio.File] = None,
+        meta: Optional[Dict[str, Any]] = None,
+    ):
         have_directories = False
         have_files = False
         for f in gfiles:
-            if f.query_file_type(
-               Gio.FileQueryInfoFlags.NONE, None) == Gio.FileType.DIRECTORY:
+            if not f:
+                continue
+            file_type = f.query_file_type(Gio.FileQueryInfoFlags.NONE, None)
+            if file_type == Gio.FileType.DIRECTORY:
                 have_directories = True
             else:
                 have_files = True
@@ -450,12 +474,17 @@ class MeldWindow(Gtk.ApplicationWindow):
                    focus=False):
         tab = None
         if len(gfiles) == 1:
-            a = gfiles[0]
-            if a.query_file_type(Gio.FileQueryInfoFlags.NONE, None) == \
-                    Gio.FileType.DIRECTORY:
-                tab = self.append_vcview(a, auto_compare)
+            gfile = gfiles[0]
+            if not gfile.query_exists():
+                raise ValueError(_("Cannot compare a non-existent file"))
+
+            if not gfile or (
+                gfile.query_file_type(Gio.FileQueryInfoFlags.NONE, None)
+                == Gio.FileType.DIRECTORY
+            ):
+                tab = self.append_vcview(gfile, auto_compare)
             else:
-                self._single_file_open(a)
+                self._single_file_open(gfile)
 
         elif len(gfiles) in (2, 3):
             tab = self.append_diff(gfiles, auto_compare=auto_compare,
