@@ -26,6 +26,7 @@ infrastructure that that's actually what we opened.
 
 import configparser
 import enum
+import logging
 import os
 import sys
 import tempfile
@@ -35,6 +36,9 @@ from gi.repository import Gio, GLib, Gtk
 
 import meld.misc
 from meld.conf import _
+from meld.iohelpers import is_file_on_tmpfs
+
+log = logging.getLogger(__name__)
 
 
 class RecentType(enum.Enum):
@@ -73,14 +77,25 @@ class RecentFiles:
         The passed flags are currently ignored. In the future these are to be
         used for extra initialisation not captured by the tab itself.
         """
-        recent_type, gfiles = tab.get_comparison()
+        try:
+            recent_type, gfiles = tab.get_comparison()
+        except Exception:
+            log.warning(f'Failed to get recent comparison data for {tab}')
+            return
 
         # While Meld handles comparisons including None, recording these as
         # recently-used comparisons just isn't that sane.
-        if None in gfiles:
+        if not gfiles or None in gfiles:
+            return
+
+        if any(is_file_on_tmpfs(f) for f in gfiles):
+            log.debug("Not adding comparison because it includes tmpfs path")
             return
 
         uris = [f.get_uri() for f in gfiles]
+        if not all(uris):
+            return
+
         names = [f.get_parse_name() for f in gfiles]
 
         # If a (type, uris) comparison is already registered, then re-add
